from _typeshed import Incomplete

class AddressGateway:
    gateway: Incomplete
    config: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, params: Incomplete | None = None): ...
    def delete(self, customer_id, address_id): ...
    def find(self, customer_id, address_id): ...
    def update(self, customer_id, address_id, params: Incomplete | None = None): ...
