<p class="counter">{{ _("There will be information about how many entities will be changed.")}}</p>
<p class="spinner" style="display: none;">
    {% snippet 'bulk/snippets/svg/spinner.svg' %}
    {{ _("Loading...") }}
</p>
