from _typeshed import Incomplete
from typing import NamedTuple

class URLComponents(NamedTuple):
    scheme: Incomplete
    netloc: Incomplete
    url: Incomplete
    params: Incomplete
    query: Incomplete
    fragment: Incomplete

def create_ipam_pool(*args, **kwargs) -> None: ...
def create_ipam_config(*args, **kwargs) -> None: ...
def decode_json_header(header): ...
def compare_version(v1, v2): ...
def version_lt(v1, v2): ...
def version_gte(v1, v2): ...
def convert_port_bindings(port_bindings): ...
def convert_volume_binds(binds): ...
def convert_tmpfs_mounts(tmpfs): ...
def convert_service_networks(networks): ...
def parse_repository_tag(repo_name): ...
def parse_host(addr, is_win32: bool = False, tls: bool = False): ...
def parse_devices(devices): ...
def kwargs_from_env(environment: Incomplete | None = None): ...
def convert_filters(filters): ...
def datetime_to_timestamp(dt): ...
def parse_bytes(s): ...
def normalize_links(links): ...
def parse_env_file(env_file): ...
def split_command(command): ...
def format_environment(environment): ...
def format_extra_hosts(extra_hosts, task: bool = False): ...
def create_host_config(self, *args, **kwargs) -> None: ...
