{% import 'macros/form.html' as form %}
{% import 'bulk/form.html' as bulk_form %}

<div class="filter-item bulk-fieldset-item">
    {{ form.select('entity_field', label=_('Entity Field'), options=[], error=errors.entity_field, classes=["bulk-field-select"]) }}
    {{ bulk_form.select('operator', label=_('Operator'), options=h.bulk_operator_options(), error=errors.operator, classes=["bulk-operator-select"]) }}
    {{ form.input('value', label=_('Value'), error=errors.value, placeholder="Field value", classes=["bulk-value-input"]) }}

    <button type="button" class="btn btn-light btn-lg btn-item-remove"><i class="fa fa-trash"></i></button>
</div>
