OP_IS = "is"
OP_IS_NOT = "is_not"
OP_CONTAINS = "contains"
OP_DOES_NOT_CONTAIN = "does_not_contain"
OP_STARTS_WITH = "starts_with"
OP_ENDS_WITH = "ends_with"
OP_IS_EMPTY = "is_empty"
OP_IS_NOT_EMPTY = "is_not_empty"

GLOBAL_AND = "AND"
GLOBAL_OR = "OR"
