def nullTransform(): ...
def translate(dx, dy): ...
def scale(sx, sy): ...
def rotate(angle): ...
def skewX(angle): ...
def skewY(angle): ...
def mmult(A, B): ...
def inverse(A): ...
def zTransformPoint(A, v): ...
def transformPoint(A, v): ...
def transformPoints(matrix, V): ...
def zTransformPoints(matrix, V): ...
